@@ -7,6 +7,8 @@
  */
 
 use MediaWiki\Logger\LoggerFactory;
+use MediaWiki\MediaWikiServices;
+use Psr\Log\LoggerInterface;
 
 /**
  * Converts LaTeX to MathML using the mathoid-server
@@ -19,6 +21,9 @@ class MathMathML extends MathRenderer {
 	protected $allowedRootElements = [];
 	protected $hosts;
 
+	/** @var LoggerInterface */
+	private $logger;
+
 	/** @var bool if false MathML output is not validated */
 	private $XMLValidation = true;
 
@@ -30,6 +35,7 @@ class MathMathML extends MathRenderer {
 	/** @var string|bool */
 	private $pngPath = false;
 
+	/** @var string|null */
 	private $mathoidStyle;
 
 	public function __construct( $tex = '', $params = [] ) {
@@ -50,6 +56,7 @@ class MathMathML extends MathRenderer {
 			// default preserve the (broken) layout as it was
 			$this->tex = '{\\displaystyle ' . $tex . '}';
 		}
+		$this->logger = LoggerFactory::getInstance( 'Math' );
 	}
 
 	/**
@@ -59,13 +66,22 @@ class MathMathML extends MathRenderer {
 		parent::addTrackingCategories( $parser );
 		if ( $this->hasWarnings() ) {
 			foreach ( $this->warnings as $warning ) {
-				if ( isset( $warning->type ) && $warning->type === 'mhchem-deprecation' ) {
-					$parser->addTrackingCategory( 'math-tracking-category-mhchem-deprecation' );
+				if ( isset( $warning->type ) ) {
+					switch ( $warning->type ) {
+						case 'mhchem-deprecation':
+							$parser->addTrackingCategory( 'math-tracking-category-mhchem-deprecation' );
+							break;
+						case 'texvc-deprecation':
+							$parser->addTrackingCategory( 'math-tracking-category-texvc-deprecation' );
+					}
 				}
 			}
 		}
 	}
 
+	/**
+	 * @param array[] $tags
+	 */
 	public static function batchEvaluate( array $tags ) {
 		$rbis = [];
 		foreach ( $tags as $key => $tag ) {
@@ -148,7 +164,7 @@ class MathMathML extends MathRenderer {
 		} catch ( Exception $e ) {
 			$this->lastError = $this->getError( 'math_mathoid_error',
 				$wgMathFullRestbaseURL, $e->getMessage() );
-			LoggerFactory::getInstance( 'Math' )->error( $e->getMessage(), [ $e, $this ] );
+			$this->logger->error( $e->getMessage(), [ $e, $this ] );
 			return false;
 		}
 	}
@@ -158,28 +174,27 @@ class MathMathML extends MathRenderer {
 	 * @return bool
 	 */
 	private function renderingRequired() {
-		$logger = LoggerFactory::getInstance( 'Math' );
 		if ( $this->isPurge() ) {
-			$logger->debug( 'Rerendering was requested.' );
+			$this->logger->debug( 'Rerendering was requested.' );
 			return true;
 		} else {
 			$dbres = $this->isInDatabase();
 			if ( $dbres ) {
 				if ( $this->isValidMathML( $this->getMathml() ) ) {
-					$logger->debug( 'Valid MathML entry found in database.' );
+					$this->logger->debug( 'Valid MathML entry found in database.' );
 					if ( $this->getSvg( 'cached' ) ) {
-						$logger->debug( 'SVG-fallback found in database.' );
+						$this->logger->debug( 'SVG-fallback found in database.' );
 						return false;
 					} else {
-						$logger->debug( 'SVG-fallback missing.' );
+						$this->logger->debug( 'SVG-fallback missing.' );
 						return true;
 					}
 				} else {
-					$logger->debug( 'Malformatted entry found in database' );
+					$this->logger->debug( 'Malformatted entry found in database' );
 					return true;
 				}
 			} else {
-				$logger->debug( 'No entry found in database.' );
+				$this->logger->debug( 'No entry found in database.' );
 				return true;
 			}
 		}
@@ -195,7 +210,7 @@ class MathMathML extends MathRenderer {
 	 * @param string $post the encoded post request
 	 * @param mixed &$res the result
 	 * @param mixed &$error the formatted error message or null
-	 * @param String $httpRequestClass class name of MWHttpRequest (needed for testing only)
+	 * @param string $httpRequestClass class name of MWHttpRequest (needed for testing only)
 	 * @return bool success
 	 */
 	public function makeRequest(
@@ -223,7 +238,7 @@ class MathMathML extends MathRenderer {
 			if ( $status->hasMessage( 'http-timed-out' ) ) {
 				$error = $this->getError( 'math_timeout', $this->getModeStr(), $host );
 				$res = false;
-				LoggerFactory::getInstance( 'Math' )->warning( 'Timeout:' . var_export( [
+				$this->logger->warning( 'Timeout:' . var_export( [
 						'post' => $post,
 						'host' => $host,
 						'timeout' => $wgMathLaTeXMLTimeout
@@ -234,7 +249,7 @@ class MathMathML extends MathRenderer {
 				$error =
 					$this->getError( 'math_invalidresponse', $this->getModeStr(), $host, $errormsg,
 						$this->getModeStr() );
-				LoggerFactory::getInstance( 'Math' )->warning( 'NoResponse:' . var_export( [
+				$this->logger->warning( 'NoResponse:' . var_export( [
 						'post' => $post,
 						'host' => $host,
 						'errormsg' => $errormsg
@@ -259,7 +274,7 @@ class MathMathML extends MathRenderer {
 		} else {
 			$host = $this->hosts;
 		}
-		LoggerFactory::getInstance( 'Math' )->debug( 'Picking host ' . $host );
+		$this->logger->debug( 'Picking host ' . $host );
 		return $host;
 	}
 
@@ -279,7 +294,7 @@ class MathMathML extends MathRenderer {
 		} else {
 			throw new MWException( 'Internal error: Restbase should be used for tex rendering' );
 		}
-		LoggerFactory::getInstance( 'Math' )->debug( 'Get post data: ' . $out );
+		$this->logger->debug( 'Get post data: ' . $out );
 		return $out;
 	}
 
@@ -297,6 +312,7 @@ class MathMathML extends MathRenderer {
 		$this->lastError = '';
 		$requestResult = $this->makeRequest( $host, $post, $res, $this->lastError );
 		if ( $requestResult ) {
+			// @phan-suppress-next-line PhanTypeMismatchArgumentInternal
 			$jsonResult = json_decode( $res );
 			if ( $jsonResult && json_last_error() === JSON_ERROR_NONE ) {
 				if ( $jsonResult->success ) {
@@ -308,7 +324,7 @@ class MathMathML extends MathRenderer {
 						$log = wfMessage( 'math_unknown_error' )->inContentLanguage()->escaped();
 					}
 					$this->lastError = $this->getError( 'math_mathoid_error', $host, $log );
-					LoggerFactory::getInstance( 'Math' )->warning(
+					$this->logger->warning(
 						'Mathoid conversion error:' . var_export( [
 							'post' => $post,
 							'host' => $host,
@@ -318,7 +334,7 @@ class MathMathML extends MathRenderer {
 				}
 			} else {
 				$this->lastError = $this->getError( 'math_invalidjson', $host );
-				LoggerFactory::getInstance( 'Math' )->error(
+				$this->logger->error(
 					'MathML InvalidJSON:' . var_export( [
 						'post' => $post,
 						'host' => $host,
@@ -346,7 +362,7 @@ class MathMathML extends MathRenderer {
 
 		$xmlObject = new XmlTypeCheck( $XML, null, false );
 		if ( !$xmlObject->wellFormed ) {
-			LoggerFactory::getInstance( 'Math' )->error(
+			$this->logger->error(
 				'XML validation error: ' . var_export( $XML, true ) );
 		} else {
 			$name = $xmlObject->getRootElement();
@@ -355,7 +371,7 @@ class MathMathML extends MathRenderer {
 			if ( in_array( $localName, $this->getAllowedRootElements() ) ) {
 				$out = true;
 			} else {
-				LoggerFactory::getInstance( 'Math' )->error( "Got wrong root element: $name" );
+				$this->logger->error( "Got wrong root element: $name" );
 			}
 		}
 		return $out;
@@ -471,6 +487,8 @@ class MathMathML extends MathRenderer {
 	 * @return string Html output that is embedded in the page
 	 */
 	public function getHtmlOutput() {
+		$config = MediaWikiServices::getInstance()->getMainConfig();
+		$enableLinks = $config->get( "MathEnableFormulaLinks" );
 		if ( $this->getMathStyle() == 'display' ) {
 			$element = 'div';
 		} else {
@@ -480,10 +498,18 @@ class MathMathML extends MathRenderer {
 		if ( $this->getID() !== '' ) {
 			$attribs['id'] = $this->getID();
 		}
+		$hyperlink = null;
 		if ( isset( $this->params['qid'] ) && preg_match( '/Q\d+/', $this->params['qid'] ) ) {
 			$attribs['data-qid'] = $this->params['qid'];
+			// TODO: SpecialPage::getTitleFor uses the depcrated method Title::newFromTitleValue and
+			// declares a never thrown MWException. Once this SpecialPage is updated, update the next line.
+			$titleObj = Title::newFromLinkTarget( SpecialPage::getTitleValueFor( 'MathWikibase' ) );
+			$hyperlink = $titleObj->getLocalURL( [ 'qid' => $this->params['qid'] ] );
 		}
 		$output = Html::openElement( $element, $attribs );
+		if ( $hyperlink && $enableLinks ) {
+			$output .= Html::openElement( 'a', [ 'href' => $hyperlink, 'style' => 'color:inherit;' ] );
+		}
 		// MathML has to be wrapped into a div or span in order to be able to hide it.
 		// Remove displayStyle attributes set by the MathML converter
 		// (Beginning from Mathoid 0.2.5 block is the default layout.)
@@ -497,6 +523,9 @@ class MathMathML extends MathRenderer {
 			'class' => $this->getClassName(), 'style' => 'display: none;'
 		], $mml );
 		$output .= $this->getFallbackImage();
+		if ( $hyperlink && $enableLinks ) {
+			$output .= Html::closeElement( 'a' );
+		}
 		$output .= Html::closeElement( $element );
 		return $output;
 	}
@@ -547,7 +576,7 @@ class MathMathML extends MathRenderer {
 					$this->setSvg( $jsonResult->svg );
 				}
 			} else {
-				LoggerFactory::getInstance( 'Math' )->error(
+				$this->logger->error(
 					'Missing SVG property in JSON result.' );
 			}
 			if ( $this->getMode() != 'latexml' && $this->inputType != 'pmml' ) {
@@ -569,8 +598,7 @@ class MathMathML extends MathRenderer {
 	 */
 	protected function isEmpty() {
 		if ( $this->userInputTex === '' ) {
-			LoggerFactory::getInstance( 'Math' )
-				->debug( 'Rendering was requested, but no TeX string is specified.' );
+			$this->logger->debug( 'Rendering was requested, but no TeX string is specified.' );
 			$this->lastError = $this->getError( 'math_empty_tex' );
 			return true;
 		}
